#include <ti/devices/msp432p4xx/driverlib/driverlib.h>
#include <stdint.h>
#include <stdbool.h>
#include <stdio.h>

// Variable iniciada
volatile uint32_t i;

// Variables iniciades
uint32_t fotocelula = 0;
uint32_t fotocelulaAnterior = 0;
uint32_t nivellTinta = 0;
uint32_t nivellTintaAnterior = 0;
uint32_t bombaPurga = 0;
uint32_t bombaPurgaAnterior = 0;
uint32_t electrovalvula = 0;
uint32_t electrovalvulaAnterior = 0;
uint32_t botoReset = 0;
uint32_t botoResetAnterior = 0;
uint32_t bombaTinta = 0;
uint32_t bombaTintaAnterior = 0;
uint32_t botoPurga = 0;
uint32_t botoPurgaAnterior = 0;

// Creaci d'array amb valors.
uint32_t valors[7] = {49,50,51,52,53,54,55};

// Declaraci de funcions.
void configInputs(); // Funci per configurar els Inputs de la MSP432.
void configLeds(); // Funci per configurar els Led's.
void delay(); // Configuraci del temps que est encs el Led.
void ledVermellNormal(); // Funci que controla el Led Vermell Normal.
void ledVermellRGB(); // Funci que controla el Led Vermell RGB.
void ledVerdRGB(); // Funci que controla el Led Verd RGB.
void ledBlauRGB(); // Funci que controla el Led Blau RGB.
void transmetreUART(uint32_t eusci, uint32_t valor); // Funci per enviar el valor per UART.

// Configuraci Port Serie
const eUSCI_UART_Config uartConfig =
{
    EUSCI_A_UART_CLOCKSOURCE_SMCLK,          // SMCLK Clock Source
    78,                                     // BRDIV = 78
    2,                                       // UCxBRF = 2
    0,                                       // UCxBRS = 0
    EUSCI_A_UART_NO_PARITY,                  // No Parity
    EUSCI_A_UART_LSB_FIRST,                  // MSB First
    EUSCI_A_UART_ONE_STOP_BIT,               // One stop bit
    EUSCI_A_UART_MODE,                       // UART mode
    EUSCI_A_UART_OVERSAMPLING_BAUDRATE_GENERATION  // Oversampling
};

int main(void)
{
    WDT_A->CTL = WDT_A_CTL_PW | WDT_A_CTL_HOLD; // Aturem el temporitzador del watchdog.

    MAP_WDT_A_holdTimer();

    configInputs(); // Funci per configurar els Inputs de la MSP432.
    configLeds(); // Funci per configurar els Led's.

    // Configurem els ports UART.
    GPIO_setAsPeripheralModuleFunctionInputPin(GPIO_PORT_P3, GPIO_PIN2 | GPIO_PIN3, GPIO_PRIMARY_MODULE_FUNCTION);

    // Ajustem DCO (rellotge) a 12MHz
    CS_setDCOCenteredFrequency(CS_DCO_FREQUENCY_12);

    // Configurem el mdulo UART
    UART_initModule(EUSCI_A2_BASE, &uartConfig);

    // Habilitem el mdul UART
    UART_enableModule (EUSCI_A2_BASE);

    while(1){
        // Analitzem cada senyal el seu estat i la guardem a una variable.
        fotocelula = GPIO_getInputPinValue(GPIO_PORT_P3, GPIO_PIN0);
        nivellTinta = GPIO_getInputPinValue(GPIO_PORT_P3, GPIO_PIN5);
        bombaPurga = GPIO_getInputPinValue(GPIO_PORT_P3, GPIO_PIN6);
        electrovalvula = GPIO_getInputPinValue(GPIO_PORT_P3, GPIO_PIN7);
        botoReset = GPIO_getInputPinValue(GPIO_PORT_P5, GPIO_PIN0);
        bombaTinta = GPIO_getInputPinValue(GPIO_PORT_P5, GPIO_PIN1);
        botoPurga = GPIO_getInputPinValue(GPIO_PORT_P5, GPIO_PIN2);

        // Analitzem el flanc de la senyal de la fotocelula.
        if (fotocelula == 0 && fotocelulaAnterior == 1)
        {
            ledVermellNormal(); // Funci que controla el Led Vermell Normal.
            transmetreUART(EUSCI_A2_BASE, valors[0]); // Funci per enviar el valor per UART.
        }
        fotocelulaAnterior = fotocelula;

        // Analitzem el flanc de la senyal del nivell de tinta.
        if (nivellTinta == 0 && nivellTintaAnterior == 1)
        {
            ledVermellNormal(); // Funci que controla el Led Vermell Normal.
            transmetreUART(EUSCI_A2_BASE, valors[1]); // Funci per enviar el valor per UART.
        }
        nivellTintaAnterior = nivellTinta;

        // Analitzem el flanc de la senyal de la bomba de purga.
        if (bombaPurga == 0 && bombaPurgaAnterior == 1)
        {
            ledVermellRGB(); // Funci que controla el Led Vermell RGB.
            transmetreUART(EUSCI_A2_BASE, valors[2]); // Funci per enviar el valor per UART.
        }
        bombaPurgaAnterior = bombaPurga;

        // Analitzem el flanc de la senyal de la electrovalvula.
        if (electrovalvula == 0 && electrovalvulaAnterior == 1)
        {
            ledVerdRGB(); // Funci que controla el Led Verd RGB.
            transmetreUART(EUSCI_A2_BASE, valors[3]); // Funci per enviar el valor per UART.
        }
        electrovalvulaAnterior = electrovalvula;

        // Analitzem el flanc de la senyal del bot de reset.
        if (botoReset == 1 && botoResetAnterior == 0)
        {
            ledBlauRGB(); // Funci que controla el Led Blau RGB.
            transmetreUART(EUSCI_A2_BASE, valors[4]); // Funci per enviar el valor per UART.
        }
        botoResetAnterior = botoReset;

        // Analitzem el flanc de la senyal de la bomba de tinta.
        if (bombaTinta == 0 && bombaTintaAnterior == 1)
        {
            ledVerdRGB(); // Funci que controla el Led Verd RGB.
            transmetreUART(EUSCI_A2_BASE, valors[5]); // Funci per enviar el valor per UART.
        }
        bombaTintaAnterior = bombaTinta;

        // Analitzem el flanc de la senyal del bot de purga.
        if (botoPurga == 1 && botoPurgaAnterior == 0)
        {
            ledVermellNormal(); // Funci que controla el Led Vermell RGB.
            transmetreUART(EUSCI_A2_BASE, valors[6]); // Funci per enviar el valor per UART.
        }
        botoPurgaAnterior = botoPurga;
    }
}

// Funci per configurar els Inputs de la MSP432.
void configInputs()
{
    // Configurem les entrades de les senyals que volem monitoritzar.
    GPIO_setAsInputPin(GPIO_PORT_P3, GPIO_PIN0);
    GPIO_setAsInputPin(GPIO_PORT_P3, GPIO_PIN5);
    GPIO_setAsInputPin(GPIO_PORT_P3, GPIO_PIN6);
    GPIO_setAsInputPin(GPIO_PORT_P3, GPIO_PIN7);
    GPIO_setAsInputPin(GPIO_PORT_P5, GPIO_PIN0);
    GPIO_setAsInputPin(GPIO_PORT_P5, GPIO_PIN1);
    GPIO_setAsInputPin(GPIO_PORT_P5, GPIO_PIN2);
}

// Funci per configurar els Led's.
void configLeds()
{
    // LED Vermell normal.
    GPIO_setAsOutputPin(GPIO_PORT_P1, GPIO_PIN0); // Configurem els Pins de sortida
    GPIO_setOutputLowOnPin(GPIO_PORT_P1, GPIO_PIN0); // Configurem l'encesa del Led.

    // LED Vermell RGB.
    GPIO_setAsOutputPin(GPIO_PORT_P2, GPIO_PIN0); // Configurem els Pins de sortida
    GPIO_setOutputLowOnPin(GPIO_PORT_P2, GPIO_PIN0); // Configurem l'encesa del Led.

    // LED Verd RGB.
    GPIO_setAsOutputPin(GPIO_PORT_P2, GPIO_PIN1); // Configurem els Pins de sortida
    GPIO_setOutputLowOnPin(GPIO_PORT_P2, GPIO_PIN1); // Configurem l'encesa del Led.

    // LED Blau RGB.
    GPIO_setAsOutputPin(GPIO_PORT_P2, GPIO_PIN2); // Configurem els Pins de sortida
    GPIO_setOutputLowOnPin(GPIO_PORT_P2, GPIO_PIN2); // Configurem l'encesa del Led.
}

// Funci que controla el Led Vermell Normal.
void ledVermellNormal()
{
    GPIO_setOutputHighOnPin(GPIO_PORT_P1, GPIO_PIN0); // Encenem el LED Vermell Normal.
    delay(); //Delay
    GPIO_setOutputLowOnPin(GPIO_PORT_P1, GPIO_PIN0); // Apaguem el LED.
}

// Funci que controla el Led Vermell RGB.
void ledVermellRGB()
{
    GPIO_setOutputHighOnPin(GPIO_PORT_P2, GPIO_PIN0); // Encenem el LED Vermell RGB.
    delay(); //Delay
    GPIO_setOutputLowOnPin(GPIO_PORT_P2, GPIO_PIN0); // Apaguem el LED.
}

// Funci que controla el Led Verd RGB.
void ledVerdRGB()
{
    GPIO_setOutputHighOnPin(GPIO_PORT_P2, GPIO_PIN1); // Encenem el LED Verd RGB.
    delay(); //Delay
    GPIO_setOutputLowOnPin(GPIO_PORT_P2, GPIO_PIN1); // Apaguem el LED.
}

// Funci que controla el Led Blau RGB.
void ledBlauRGB()
{
    GPIO_setOutputHighOnPin(GPIO_PORT_P2, GPIO_PIN2); // Encenem el LED Blau RGB.
    delay(); //Delay
    GPIO_setOutputLowOnPin(GPIO_PORT_P2, GPIO_PIN2); // Apaguem el LED.
}

// Configuraci del temps que est encs el Led.
void delay()
{
    for(i=100000; i>0; i--); //Delay
}

// Funci per enviar el valor per UART.
void transmetreUART(uint32_t eusci, uint32_t valor)
{
    UART_transmitData(eusci, valor); // Transmissi per UART del valor asignat per la senyal.
}
